﻿using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace fblock.manager
{
    public partial class MainForm : Form
    {
        private bool connected = false;
        private IFBlockAccessorEvents events = null;
        private IFBlockAccessor accessor = null;

        private class ReportAdditionalComboHelper
        {
            private string text = "";
            private object data = null;

            public ReportAdditionalComboHelper(string text, object data)
            {
                this.text = text;
                this.data = data;
            }

            public string Text
            {
                get { return this.text; }
            }

            public object Data
            {
                get { return this.data; }
            }

            public override string ToString()
            {
                return this.Text;
            }
        }

        public MainForm(IFBlockAccessorEvents events, IFBlockAccessor accessor)
        {
            this.events = events;
            this.accessor = accessor;
            this.InitializeComponent();

            this.cmbReportAdditional.Items.AddRange(new ReportAdditionalComboHelper[]{
                new ReportAdditionalComboHelper("Argument Exception",new ArgumentException("Argument Exception")),
                new ReportAdditionalComboHelper("Argument Null Exception",new ArgumentNullException("Argument NULL Exception")),
                new ReportAdditionalComboHelper("Null Reference Exception",new NullReferenceException("NULL Reference Exception")),
                new ReportAdditionalComboHelper("Access Violation Exception",new AccessViolationException("Access Violation Exception")), 
                new ReportAdditionalComboHelper("Format Exception",new FormatException ("Format Exception")), 
                new ReportAdditionalComboHelper("Index Out Of Range Exception",new IndexOutOfRangeException ("Index Out Of Range Exception")), 
                new ReportAdditionalComboHelper("Invalid Operation Exception",new InvalidOperationException ("Invalid Operation Exception")), 
                new ReportAdditionalComboHelper("Not Implemented Exception",new NotImplementedException ("Not Implemented Exception")), 
                new ReportAdditionalComboHelper("Not Supported Exception",new NotSupportedException ("Not Supported Exception")), 
                new ReportAdditionalComboHelper("Out Of Memory Exception",new OutOfMemoryException ("Out Of Memory Exception")), 
                new ReportAdditionalComboHelper("Overflow Exception",new OverflowException ("Overflow Exception")), 
                new ReportAdditionalComboHelper("System Exception",new SystemException ("System Exception")), 
            });
            this.cmbReportAdditional.SelectedIndex = 0;
        }

        public int FBlockID
        {
            // Get this from a real funcktion block!
            get { return 0x4711; }
        }

        public int InstanceID
        {
            get { return Convert.ToInt32(this.numInstanceID.Value); }
        }

        public bool Connect()
        {
            if (!this.connected)
            {
                this.btnConnect.Text = "&Disconnect";
                this.numInstanceID.Enabled = false;

                this.connected = true;

                // Ensure event firing after changing value of "connected"
                if (this.events != null)
                {
                    this.events.FireConnectedEvent();
                }
            }
            return this.connected;
        }

        public bool Disconnect()
        {
            if (this.connected)
            {
                this.btnConnect.Text = "&Connect";
                this.numInstanceID.Enabled = true;

                this.connected = false;

                // Ensure event firing after changing value of "connected"
                if (this.events != null)
                {
                    this.events.FireDisconnectedEvent();
                }
            }
            return !this.connected;
        }

        private void btnConnect_Click(object sender, EventArgs args)
        {
            if (this.connected)
            {
                this.Disconnect();
            }
            else
            {
                this.Connect();
            }
        }

        private void numInstanceID_ValueChanged(object sender, EventArgs args)
        {
            if (this.events != null)
            {
                this.events.FireInstanceIDChangedEvent(this.InstanceID);
            }
        }

        private void btnReportApply_Click(object sender, EventArgs args)
        {
            if (this.accessor != null && this.accessor.ReportSink!= null)
            {
                if (this.radReportTypeMessage.Checked)
                {
                    this.accessor.ReportSink.Message(
                        this.txtReportDescription.Text,
                        this.txtReportSource.Text,
                        this.txtReportCategory.Text,
                        ((ReportAdditionalComboHelper)this.cmbReportAdditional.SelectedItem).Data
                        );
                }
                if (this.radReportTypeWarning.Checked)
                {
                    this.accessor.ReportSink.Warning(
                        this.txtReportDescription.Text,
                        this.txtReportSource.Text,
                        this.txtReportCategory.Text,
                        ((ReportAdditionalComboHelper)this.cmbReportAdditional.SelectedItem).Data
                        );
                }
                if (this.radReportTypeError.Checked)
                {
                    this.accessor.ReportSink.Error(
                        this.txtReportDescription.Text,
                        this.txtReportSource.Text,
                        this.txtReportCategory.Text,
                        ((ReportAdditionalComboHelper)this.cmbReportAdditional.SelectedItem).Data
                        );
                }
                if (this.radReportTypeDebug.Checked)
                {
                    this.accessor.ReportSink.Debug(
                        this.txtReportDescription.Text,
                        this.txtReportSource.Text,
                        this.txtReportCategory.Text,
                        ((ReportAdditionalComboHelper)this.cmbReportAdditional.SelectedItem).Data
                        );
                }
            }
        }

        private void btnBoundsMove_Click(object sender, EventArgs args)
        {
            System.Diagnostics.Debug.WriteLine("Before: " + this.Location.ToString());

            this.Location = new Point(this.Location.X + Screen.PrimaryScreen.Bounds.Size.Width, this.Location.Y);

            System.Diagnostics.Debug.WriteLine("After: " + this.Location.ToString());
        }
    }
}
