﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;

namespace fblock.manager
{
    public class DefaultFBlockAccessorEvents : IFBlockAccessorEvents
    {
        #region IFBlockAccessorEvents member definition

        public event EventHandler<EventArgs> Connected;

        public event EventHandler<EventArgs> Disconnected;

        public event EventHandler<InstanceIDChangedEventArgs> InstanceIDChanged;

        public event EventHandler<ShowUIChangedEventArgs> ShowUIChanged;

        public event EventHandler<EventArgs> LocationChanged;

        #endregion // IFBlockAccessorEvents member definition

        #region IFBlockAccessorEvents delegate definition

        protected delegate void ConnectedEventHandler();

        protected delegate void DisconnectedEventHandler();

        protected delegate void InstanceIDChangedEventHandler(int instanceID);

        protected delegate void ShowUIChangedEventHandler(bool showUI, ShowUIChangedEventArgs.ShowState showState);

        protected delegate void LocationChangedEventHandler();

        #endregion // IFBlockAccessorEvents delegate definition

        #region IFBlockAccessorEvents fire event function implementation

        public void FireConnectedEvent()
        {
            EventHandler<EventArgs> eventHandler = Connected;
            if (eventHandler != null)
            {
                eventHandler(this, new EventArgs());
            }
        }

        public void FireDisconnectedEvent()
        {
            EventHandler<EventArgs> eventHandler = Disconnected;
            if (eventHandler != null)
            {
                eventHandler(this, new EventArgs());
            }
        }

        public void FireInstanceIDChangedEvent(int instanceID)
        {
            EventHandler<InstanceIDChangedEventArgs> eventHandler = InstanceIDChanged;
            if (eventHandler != null)
            {
                eventHandler(this, new InstanceIDChangedEventArgs(instanceID));
            }
        }

        public void FireShowUIChangedEvent(bool showUI, ShowUIChangedEventArgs.ShowState showState)
        {
            EventHandler<ShowUIChangedEventArgs> eventHandler = ShowUIChanged;
            if (eventHandler != null)
            {
                eventHandler(this, new ShowUIChangedEventArgs(showUI, showState));
            }
        }

        public void FireLocationChangedEvent()
        {
            EventHandler<EventArgs> eventHandler = LocationChanged;
            if (eventHandler != null)
            {
                eventHandler(this, new EventArgs());
            }
        }

        #endregion // IFBlockAccessorEvents fire event function implementation
    }
}

// EOF