﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Windows.Forms;

namespace fblock.manager
{
    public interface IFBlockAccessor
    {
        // Gets the readable name of this FBlock/Shadow.
        string Name { get; }
        
        // Gets the type of this FBlock/Shadow.
        string Category { get; }

        // Gets the description of this FBlock/Shadow.
        string Description { get; }

        // Gets version information about this FBlock/Shadow.
        string Version { get; }

        // Gets information about the catalog used with this FBlock/Shadow.
        string Catalog { get; }

        // Gets the function block ID of this FBlock/Shadow.
        int FBlockID { get; }

        // Gets the instance ID of this FBlock/Shadow.
        int InstanceID { get; }

        // Gets the connected state of this FBlock/Shadow.
        bool IsConnected { get; }

        // Gets the reporter sink to be used to report messages.
        IReportSink ReportSink { get; }

        // Gets and sets the UI visibility of this FBlock/Shadow.
        bool ShowUI { get; set; }

        // Gets this FBlock's/Shadow's main from.
        Form MainForm { get; }

        // Initializes this FBlock/Shadow instance.
        bool Initialize(IReportSink reportSinkPanel);

        // Cleans this FBlock/Shadow instance and frees all resources.
        bool Cleanup();

        // Connects this FBlock/Shadow to the MOST bus.
        bool Connect();

        // Disconnects this FBlock/Shadow from the MOST bus.
        bool Disconnect();

        // Loads initial states and settings from a configuration.
        bool LoadConfiguration();

        // Saves current states and settings into a configuration.
        bool SaveConfiguration();
    }
}

// EOF