﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;

namespace fblock.manager
{
    public interface IFBlockAccessorEvents
    {
        // This event should occur when a new connection could be made.
        event EventHandler<EventArgs> Connected;

        // This event should occur when an existing connection was closed.
        event EventHandler<EventArgs> Disconnected;

        // This event should occur when the instance ID has been changed.
        event EventHandler<InstanceIDChangedEventArgs> InstanceIDChanged;

        // This event should occur when the UI visibility has been changed.
        event EventHandler<ShowUIChangedEventArgs> ShowUIChanged;

        // This event should occur when the UI's location has been changed.
        event EventHandler<EventArgs> LocationChanged;

        // Fires Connected event.
        void FireConnectedEvent();

        // Fires Disconnected event.
        void FireDisconnectedEvent();

        // Fires InstanceIDChanged event.
        void FireInstanceIDChangedEvent(int instanceID);

        // Fires ShowUIChanged event.
        void FireShowUIChangedEvent(bool showUI, ShowUIChangedEventArgs.ShowState showState);

        // Fires LocationChanged event.
        void FireLocationChangedEvent();
    }
}

// EOF