﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace fblock.manager
{
    partial class AboutDialog : Form
    {
        public AboutDialog()
        {
            this.InitializeComponent();
            this.Text = String.Format("About {0}", AboutDialog.AssemblyTitle);
            this.lblProduct.Text = AboutDialog.AssemblyProduct;
            this.lblVersion.Text = AboutDialog.AssemblyVersion;
            this.lblCopyright.Text = AboutDialog.AssemblyCopyright;
            this.txtDescription.Text = AboutDialog.AssemblyDescription;

            this.lnkIconAuthor.Text = "Icons by Mark James";
            this.lnkIconAuthor.Links.Add(9, 10, "http://www.famfamfam.com/");
        }

        #region Assembly Attribute Accessors

        public static string AssemblyTitle
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
                if (attributes.Length > 0)
                {
                    AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
                    if (titleAttribute.Title != "")
                    {
                        return titleAttribute.Title;
                    }
                }
                return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
            }
        }

        public static string AssemblyVersion
        {
            get
            {
                if (Assembly.GetExecutingAssembly().GetName().Version.Revision == 0)
                {
                    return String.Format("Version {0}.{1}.{2}",
                        Assembly.GetExecutingAssembly().GetName().Version.Major,
                        Assembly.GetExecutingAssembly().GetName().Version.Minor,
                        Assembly.GetExecutingAssembly().GetName().Version.Build
                    );
                }
                else
                {
                    return String.Format("Version {0}.{1}.{2} Build {3}",
                        Assembly.GetExecutingAssembly().GetName().Version.Major,
                        Assembly.GetExecutingAssembly().GetName().Version.Minor,
                        Assembly.GetExecutingAssembly().GetName().Version.Build,
                        Assembly.GetExecutingAssembly().GetName().Version.Revision
                    );
                }
            }
        }

        public static string AssemblyDescription
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false);
                if (attributes.Length == 0)
                {
                    return "";
                }
                return ((AssemblyDescriptionAttribute)attributes[0]).Description;
            }
        }

        public static string AssemblyProduct
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
                if (attributes.Length == 0)
                {
                    return "";
                }
                return ((AssemblyProductAttribute)attributes[0]).Product;
            }
        }

        public static string AssemblyCopyright
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
                if (attributes.Length == 0)
                {
                    return "";
                }
                return ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
            }
        }

        public static string AssemblyCompany
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCompanyAttribute), false);
                if (attributes.Length == 0)
                {
                    return "";
                }
                return ((AssemblyCompanyAttribute)attributes[0]).Company;
            }
        }

        #endregion // Assembly attribute accessors

        private void lnkIconAuthor_LinkClicked(object sender, LinkLabelLinkClickedEventArgs args)
        {
            args.Link.Visited = true;
            System.Diagnostics.Process.Start(args.Link.LinkData.ToString());
        }
    }
}

// EOF