﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Drawing;
using System.Windows.Forms;

namespace fblock.manager
{
    public class ColorButton : Button
    {
        private Color color;
        private bool border;

        public ColorButton()
            : base()
        {
            base.Padding = new Padding(5);
            base.Text = "<empty>";
            this.Color = Color.White;
            this.Border = true;
        }

        public ColorButton(Color color)
            : this()
        {
            this.color = color;
        }

        public ColorButton(Color color, bool border)
            : this(color)
        {
            this.border = border;
        }

        public Color Color
        {
            get { return this.color; }
            set { this.color = value; }
        }

        public bool Border
        {
            get { return this.border; }
            set { this.border = value; }
        }

        // Override to avoid!
        public new string Text
        {
            get { return ""; }
            set { }
        }

        protected override void OnPaint(PaintEventArgs args)
        {
            base.OnPaint(args);

            if (!this.color.IsEmpty)
            {
                Rectangle rect = this.ClientRectangle;
                rect.X += this.Padding.Left;
                rect.Y += this.Padding.Top;
                rect.Width -= (this.Padding.Left + this.Padding.Right + 1);
                rect.Height -= (this.Padding.Top + this.Padding.Bottom + 1);

                SolidBrush brush = new SolidBrush(this.color);
                args.Graphics.FillRectangle(brush, rect);
                brush.Dispose();

                if (this.border)
                {
                    args.Graphics.DrawRectangle(Pens.Black, rect);
                }
            }
        }
    }
}
