﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Reflection;
using System.Windows.Forms;

namespace fblock.manager
{
    [Serializable]
    public class FBlockInvoker
    {
        private IReportSink reporter = null;
        private string assemblyName;
        private Type type = null;
        private object pObject = null;

        public FBlockInvoker(string assemblyName, IReportSink reporter)
        {
            // Save given report sink to be used.
            this.reporter = reporter;

            // Save given assembly name.
            this.assemblyName = assemblyName;

            // Try loading given assembly.
            this.InvokeCreateInstance();
        }

        #region FBlockInvoker helper function implementation

        public bool IsValid
        {
            get { return (this.type != null && this.pObject != null); }
        }

        public bool HasEvents
        {
            get
            {
                if (this.type != null)
                {
                    return (this.type.GetInterface("fblock.manager.IFBlockAccessorEvents") != null);
                }
                else
                {
                    return false;
                }
            }
        }

        public string AssemblyName
        {
            get { return this.assemblyName; }
        }

        public IFBlockAccessorEvents GetEvents()
        {
            if (this.HasEvents)
            {
                return (IFBlockAccessorEvents)this.pObject;
            }
            else
            {
                return null;
            }
        }

        public IFBlockAccessor GetAccessor()
        {
            return (IFBlockAccessor)this.pObject;
        }


        #endregion

        #region IFBlockAccessor property access function implementation

        public string Name
        {
            get
            {
                try
                {
                    return (string)this.InvokePropertyGet("Name", typeof(string));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Name: Result type was unexpected and differs from input type \"string\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Name: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return "";
            }
        }

        public string Category
        {
            get
            {
                try
                {
                    return (string)this.InvokePropertyGet("Category", typeof(string));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Type: Result type was unexpected and differs from input type \"string\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Type: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return "";
            }
        }

        public string Description
        {
            get
            {
                try
                {
                    return (string)this.InvokePropertyGet("Description", typeof(string));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Description: Result type was unexpected and differs from input type \"string\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Description: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return "";
            }
        }

        public string Version
        {
            get
            {
                try
                {
                    return (string)this.InvokePropertyGet("Version", typeof(string));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Version: Result type was unexpected and differs from input type \"string\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Version: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return "";
            }
        }

        public string Catalog
        {
            get
            {
                try
                {
                    return (string)this.InvokePropertyGet("Catalog", typeof(string));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Catalog: Result type was unexpected and differs from input type \"string\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("Catalog: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return "";
            }
        }

        public int FBlockID
        {
            get
            {
                try
                {
                    return (int)this.InvokePropertyGet("FBlockID", typeof(int));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("FBlockID: Result type was unexpected and differs from input type \"int\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("FBlockID: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return -1;
            }
        }

        public int InstanceID
        {
            get
            {
                try
                {
                    return (int)this.InvokePropertyGet("InstanceID", typeof(int));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("InstanceID: Result type was unexpected and differs from input type \"int\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("InstanceID: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return -1;
            }
        }

        public bool IsConnected
        {
            get
            {
                try
                {
                    return (bool)this.InvokePropertyGet("IsConnected", typeof(int));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("IsConnected: Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("IsConnected: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return false;
            }
        }

        public IReportSink ReportSink
        {
            get
            {
                try
                {
                    return (IReportSink)this.InvokePropertyGet("ReportSink", typeof(IReportSink));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("ReportSink: Result type was unexpected and differs from input type \"IReportSink\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("ReportSink: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return null;
            }
        }

        public bool ShowUI
        {
            get
            {
                try
                {
                    return (bool)this.InvokePropertyGet("ShowUI", typeof(bool));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("ShowUI: Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("ShowUI: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return false;
            }
            set
            {
                try
                {
                    this.InvokePropertySet("ShowUI", value);
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("ShowUI: " + exception.Message, "FBlockManager", "Property.Set", exception);
                    }
                }
            }
        }

        public Form MainForm
        {
            get
            {
                try
                {
                    return (Form)this.InvokePropertyGet("MainForm", typeof(Form));
                }
                catch (InvalidCastException /*exception*/ )
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("MainForm: Result type was unexpected and differs from input type \"Form\"", "FBlockManager", "Property.Get");
                    }
                }
                catch (Exception exception)
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Error("MainForm: " + exception.Message, "FBlockManager", "Property.Get", exception);
                    }
                }
                return null;
            }
        }

        #endregion

        #region IFBlockAccessor method access function implementation

        public bool Initialize(IReportSink reportSinkPanel)
        {
            try
            {
                return (bool)this.InvokeMember("Initialize", new object[] { reportSinkPanel }, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Initialize(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Initialize(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        public bool Cleanup()
        {
            try
            {
                return (bool)this.InvokeMember("Cleanup", null, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Cleanup(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Cleanup(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        public bool Connect()
        {
            try
            {
                return (bool)this.InvokeMember("Connect", null, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Connect(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Connect(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        public bool Disconnect()
        {
            try
            {
                return (bool)this.InvokeMember("Disconnect", null, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Disconnect(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Disconnect(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        public bool LoadConfiguration()
        {
            try
            {
                return (bool)this.InvokeMember("LoadConfiguration", null, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("LoadConfiguration(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("LoadConfiguration(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        public bool SaveConfiguration()
        {
            try
            {
                return (bool)this.InvokeMember("SaveConfiguration", null, typeof(bool));
            }
            catch (InvalidCastException /*exception*/ )
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("SaveConfiguration(): Result type was unexpected and differs from input type \"bool\"", "FBlockManager", "Method");
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("SaveConfiguration(): " + exception.Message, "FBlockManager", "Method", exception);
                }
            }
            return false;
        }

        #endregion

        #region FBlockInvoker private function implementation

        private object InvokePropertyGet(string propertyName, Type returnType)
        {
            if (this.IsValid)
            {
                try
                {
                    // Dynamically invoke the object
                    BindingFlags flags = BindingFlags.Default | BindingFlags.GetProperty;
                    return this.type.InvokeMember(propertyName, flags, null, this.pObject, null);
                }
                catch (Exception exception)
                {
                    throw new MemberAccessException("Call to property " + propertyName + " failed.", exception);
                }
            }
            else
            {
                throw new InvalidOperationException("Assembly mismatched or not initialized.");
            }
        }

        private void InvokePropertySet(string propertyName, object argument)
        {
            if (this.IsValid)
            {
                try
                {
                    // Dynamically invoke the object
                    BindingFlags flags = BindingFlags.Default | BindingFlags.SetProperty;
                    this.type.InvokeMember(propertyName, flags, null, this.pObject, new object[] { argument });
                }
                catch (Exception exception)
                {
                    throw new MemberAccessException("Call to property " + propertyName + " failed.", exception);
                }
            }
            else
            {
                throw new InvalidOperationException("Assembly mismatched or not set.");
            }
        }

        private object InvokeMember(string methodName, object[] arguments, Type returnType)
        {
            if (this.IsValid)
            {
                try
                {
                    // Dynamically invoke the object
                    BindingFlags flags = BindingFlags.Default | BindingFlags.InvokeMethod;
                    return this.type.InvokeMember(methodName, flags, null, this.pObject, arguments);
                }
                catch (Exception exception)
                {
                    throw new MemberAccessException("Call to member function " + methodName + " failed.", exception);
                }
            }
            else
            {
                throw new InvalidOperationException("Assembly mismatched or not set.");
            }
        }

        private void InvokeCreateInstance()
        {
            // Create test load domain to be used.
            AppDomain appDomain = AppDomain.CreateDomain("TestLoadDomain");

            try
            {
                // Try to load assebly in a different domain and fail in case of an exception
                appDomain.DoCallBack(new CrossAppDomainDelegate(CreateInstance));

                // Reload same assembly into current domain in case of no exception!
                this.CreateInstance();
            }
            finally
            {
                AppDomain.Unload(appDomain);
            }
        }

        private void CreateInstance()
        {
            try
            {
                // Reset last loaded object instance.
                this.pObject = null;
                this.type = null;

                // Try open given file and load as assembly.
                Assembly assembly = Assembly.LoadFrom(assemblyName);

                // Walk through each type in the assembly
                foreach (Type type in assembly.GetTypes())
                {
                    if (type.IsClass == true)
                    {
                        if (type.GetInterface("fblock.manager.IFBlockAccessor") != null)
                        {
                            this.pObject = Activator.CreateInstance(type);
                            this.type = type;
                        }
                    }
                }
            }
            catch (Exception exception)
            {
                // Do something else with this exception if needed
                throw exception;
            }
        }

        #endregion
    }
}

// EOF