﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using Microsoft.Win32;
using System.Reflection;

namespace fblock.manager
{
    public class FileAssociation
    {
        private static string extension = ".fblock";
        private static string programID = "FBlock.Project";
        private static string description = "FBlock Project File";
        private static string rootKey = @"Software\Classes\";
        private static string iconKey = "DefaultIcon";
        private static string openKey = @"Shell\Open\Command\";

        public static void Associate(ReportSink reporter)
        {
            RegistryKey root = null;
            RegistryKey prog = null;
            try
            {
                string exec = "\"" + Assembly.GetExecutingAssembly().Location + "\" \"%1\"";
                string icon = @"%SystemRoot%\System32\shell32.dll,-151";

                root = Registry.CurrentUser.OpenSubKey(FileAssociation.rootKey, true);
                root.CreateSubKey(FileAssociation.extension).SetValue("", FileAssociation.programID);
                root.CreateSubKey(FileAssociation.programID).SetValue("", FileAssociation.description);

                prog = root.OpenSubKey(FileAssociation.programID, true);
                prog.CreateSubKey(FileAssociation.iconKey).SetValue("", icon, RegistryValueKind.ExpandString);
                prog.CreateSubKey(FileAssociation.openKey).SetValue("", exec);

                if (reporter != null)
                {
                    reporter.Message("Project files successful associated.", "FBlockManager", "Registry");
                }
            }
            catch (Exception exception)
            {
                System.Diagnostics.Debug.WriteLine(exception.ToString());

                if (reporter != null)
                {
                    reporter.Error("Associating FBlock project files with the FBlockManager application failed!", "FBlockManager", "Registry", exception);
                }
            }
            finally
            {
                if (prog != null) { prog.Close(); }
                if (root != null) { root.Close(); }
            }
        }

        public static void Remove(ReportSink reporter)
        {
            RegistryKey root = null;
            try
            {
                root = Registry.CurrentUser.OpenSubKey(FileAssociation.rootKey, true);
                root.DeleteSubKey(FileAssociation.extension);
                root.DeleteSubKeyTree(FileAssociation.programID);

                if (reporter != null)
                {
                    reporter.Message("Association of project files successful removed.", "FBlockManager", "Registry");
                }
            }
            catch (Exception exception)
            {
                System.Diagnostics.Debug.WriteLine(exception.ToString());

                if (reporter != null)
                {
                    reporter.Error("Removing FBlock project file association failed!", "FBlockManager", "Registry", exception);
                }
            }
            finally
            {
                if (root != null) { root.Close(); }
            }
        }

        public static bool IsAssociated()
        {
            bool result = false;
            RegistryKey root = null;
            try
            {
                root = Registry.CurrentUser.OpenSubKey(FileAssociation.rootKey + FileAssociation.extension, false);
                result = (root != null);
            }
            finally
            {
                if (root != null) { root.Close(); }
            }
            return result;
        }
    }
}

