﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace fblock.manager
{
    internal static class InstanceAccessor
    {
        private const int WM_APP = 0x8000;
        public const int WM_APP_ALIVE = (WM_APP | 0x0FFF) - 1;
        public const int WM_APP_FRONT = (WM_APP | 0x0FFF) - 2;
        public const int WM_APP_CLOSE = (WM_APP | 0x0FFF) - 3;

        private const int APP_RESULT = 0x4200;
        public const int APP_ALIVE_RESULT = APP_RESULT + 1;
        public const int APP_FRONT_RESULT = APP_RESULT + 2;
        public const int APP_CLOSE_RESULT = APP_RESULT + 3;

        private class RequestHelper
        {
            private int question = 0;
            private int answer = 0;
            private bool success = false;
            private IntPtr wParam = IntPtr.Zero;
            private IntPtr lParam = IntPtr.Zero;

            public RequestHelper(int question, int answer)
                : base()
            {
                this.question = question;
                this.answer = answer;
            }

            public int Question
            {
                get { return this.question; }
            }

            public int Answer
            {
                get { return this.answer; }
            }

            public IntPtr WParam
            {
                get { return this.wParam; }
                set { this.wParam = value; }
            }

            public IntPtr LParam
            {
                get { return this.lParam; }
                set { this.lParam = value; }
            }

            public bool Success
            {
                get { return this.success; }
                set { this.success = value; }
            }
        }

        public static bool IsRunning()
        {
            RequestHelper helper = new RequestHelper(WM_APP_ALIVE, APP_ALIVE_RESULT);
            try
            {
                EnumWindows(new EnumWindowsDelegate(EnumWindowsExecution), helper);
            }
            catch (Exception /*exception*/)
            {
            }
            return helper.Success;
        }

        public static bool RequestActivate()
        {
            RequestHelper helper = new RequestHelper(WM_APP_FRONT, APP_FRONT_RESULT);
            try
            {
                EnumWindows(new EnumWindowsDelegate(EnumWindowsExecution), helper);
            }
            catch (Exception /*exception*/)
            {
            }
            return helper.Success;
        }

        public static bool RequestShutdown(bool nosave)
        {
            RequestHelper helper = new RequestHelper(WM_APP_CLOSE, APP_CLOSE_RESULT);
            try
            {
                helper.LParam = new IntPtr(Convert.ToInt32(!nosave));
                EnumWindows(new EnumWindowsDelegate(EnumWindowsExecution), helper);
            }
            catch (Exception /*exception*/)
            {
            }
            return helper.Success;
        }

        private delegate bool EnumWindowsDelegate(IntPtr hWnd, RequestHelper helper);
        private static bool EnumWindowsExecution(IntPtr hWnd, RequestHelper helper)
        {
            try
            {
                StringBuilder builder = new StringBuilder(501);
                if (0 < GetWindowText(hWnd, builder, builder.Capacity - 1))
                {
                    // ATTENTION: Also change string literal if MainForm title changes!
                    if (builder.ToString().ToLower().Contains("fblock manager"))
                    {
                        IntPtr result = SendMessage(hWnd, helper.Question, helper.WParam, helper.LParam);
                        if (result.ToInt32() == helper.Answer)
                        {
                            helper.Success = true;
                            return false; // Don't continue scanning.
                        }
                    }
                }
            }
            catch
            {
            }
            return true; //Continue scanning.
        }

        #region Win32 API function declaration

        [DllImport("user32.dll")]
        private static extern bool EnumWindows(EnumWindowsDelegate callback, RequestHelper helper);

        [DllImport("user32.dll")]
        private static extern int GetWindowText(IntPtr hWnd, StringBuilder text, int count);

        [DllImport("user32.dll")]
        private static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);

        #endregion // Win32 API function declaration
    }
}

// EOF