﻿/*
 * Copyright (C)  2010  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace fblock.manager
{
    public class ProjectSettings
    {
        private DateTime saved = DateTime.Now;
        private string author = Environment.UserName;
        private string remark = "";
        private string division = "";
        private string description = "";
        private List<ProjectEntry> projectEntrys = new List<ProjectEntry>();

        public ProjectSettings()
        {
        }

        public DateTime Saved
        {
            get { return this.saved; }
            set { this.saved = value; }
        }

        public string Author
        {
            get { return this.author; }
            set { this.author = value; }
        }

        public string Remark
        {
            get { return this.remark; }
            set { this.remark = value; }
        }

        public string Division
        {
            get { return this.division; }
            set { this.division = value; }
        }

        public string Description
        {
            get { return this.description; }
            set { this.description = value; }
        }

        public List<ProjectEntry> ProjectEntrys
        {
            get { return this.projectEntrys; }
            set { this.projectEntrys = value; }
        }
    }

    public class ProjectEntry
    {
        private string name = "";
        private string assembly = "";
        private bool showUI = false;
        private bool connected = false;
        private bool selected = false;
        private Point location = new Point(0, 0);

        public ProjectEntry()
        {
        }

        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public string Assembly
        {
            get
            {
                if (ProjectSerializer.Serializing)
                {
                    return this.assembly;
                }
                else
                {
                    return ApplicationSettings.AbsolutPath(this.assembly);
                }
            }

            set { this.assembly = ApplicationSettings.RelativePath(value); }
        }

        public bool ShowUI
        {
            get { return this.showUI; }
            set { this.showUI = value; }
        }

        public bool Connected
        {
            get { return this.connected; }
            set { this.connected = value; }
        }

        public bool Selected
        {
            get { return this.selected; }
            set { this.selected = value; }
        }

        public Point Location
        {
            get { return this.location; }
            set { this.location = value; }
        }
    }

    public class ProjectSerializer
    {
        private ProjectSettings settings = null;
        private IReportSink reporter = null;

        private static bool serializing = false;

        public static bool Serializing
        {
            get { return ProjectSerializer.serializing; }
        }

        public ProjectSerializer(IReportSink reporter)
        {
            this.reporter = reporter;
        }

        public ProjectSettings Settings
        {
            get { return this.settings; }
            set { this.settings = value; }
        }

        public bool Serialize(string filename)
        {
            TextWriter writer = null;
            ProjectSerializer.serializing = true;
            try
            {
                if (this.settings != null)
                {
                    XmlSerializer serializer = new XmlSerializer(typeof(ProjectSettings));
                    writer = new StreamWriter(filename);
                    serializer.Serialize(writer, this.Settings);

                    return true;
                }
                else
                {
                    if (this.reporter != null)
                    {
                        this.reporter.Warning("Nothing to serialize", "FBlockManager", "Serialize");
                    }
                    return false;
                }
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Serializing file " + filename + " failed", "FBlockManager", "Serialize", exception);
                }
                return false;
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
                ProjectSerializer.serializing = false;
            }
        }

        public bool Deserialize(string filename)
        {
            TextReader reader = null;
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(ProjectSettings));
                reader = new StreamReader(filename);
                this.settings = (ProjectSettings)serializer.Deserialize(reader);
                return true;
            }
            catch (Exception exception)
            {
                if (this.reporter != null)
                {
                    this.reporter.Error("Deserializing file " + filename + " failed", "FBlockManager", "Deserialize", exception);
                }
                return false;
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }
    }
}

// EOF